function [time,corrected,opt_ds] = artefact_correction(input)
% artefact_correction   Removal of the partial excitation overlap from
%                       5-pulse DEER data
%
% [time,corrected] = artefact_correction(input)
%
% input     structure of input data with fields, 
%           .t1, .trace1, .trace2, .ds are required
%
% .t1       time axis of first trace, coincides with time axis of second
%           trace
% .trace1   first trace
% .trace2   second trace
% .ds       shift of the artefact in the second trace (points), l in paper
% .tshift   approximate time shift of the artefact with respect to the main
%           signal, defaults to the center of the traces, only used for
%           searching for the artefact
% .ext_zt   external zerotime, if not given, the zero time of the artefact
%           is estimated by the algorithm
% .display  display intermediate plots and information, defaults to true
%
% Expert mode input parameters (can all be omitted, as they have defaults):
%
% .cutdata  fraction of data to be included in analysis, default: 1
%           for, e.g., 0.8 the last 20% of data points are cut off
% .t4       time axis for comparative 4-pulse DEER data, if not given,
%           comparison is omitted
% .tr4      comparative 4-pulse DEER trace, if not given, comparison is
%           omitted
% .bg_mode  background mode for modulation depth estimate, string
%           '3D', '2D', 'fractal', 'none' (for form factor input), defaults
%           to '3D', needed only for fitting 4-pulse DEER comparison
% exp_depth experimental modulation depth estimate, needed only as starting
%           parameter for fitting 4-pulse DEER comparison data, default 0.3
% fit_depth flag for fitting modulation depth in 4-pulse DEER comparison,
%           defaults to 1 (true)
% cut4p     number of data points of 4-pulse comparison trace to be
%           omitted, defaults to 0
% interp    data interpolation to 0.5 ns time increment, defaults to false, 
%           use only when you have a good reason
% sec_fit   secondary fit of artefact amplitude by balancing the corrected
%           traces, use only if you know why, defaults to false
%
% Programmer mode input parameters:
%
% .degree   polynomial degree for artefact baseline correction, default: 3
% N          number of data points in DEER data, defaults to 1024
%
% Output:
% time       time axis of corrected trace
% corrected	 corrected trace 
% opt_ds     suggested optimum shift of second trace
%
% can fail in secondary fit mode and returns empty time and corrected trace
% vectors in this case
%
% G. Jeschke, 2015,2017
% adapted by Y. Polyhach and F. Breitgoff, 2016

% Check input for completeness and set internal parameters from input or
% defaults

if ~isfield(input,'t1') || isempty(input.t1),
    fprintf(2,'ERROR: Time axis input.t1 of the first trace must be provided.\n');
    return
else
    t1 = input.t1;
end;

if ~isfield(input,'trace1') || isempty(input.trace1),
    fprintf(2,'ERROR: First trace input.trace1 must be provided.\n');
    return
else
    ffan = input.trace1;
end;

if ~isfield(input,'trace2') || isempty(input.trace2),
    fprintf(2,'ERROR: Second trace input.trace2 must be provided.\n');
    return
else
    ffbn = input.trace2;
end;

if ~isfield(input,'ds') || isempty(input.ds),
    fprintf(2,'ERROR: Artefact shift input.ds in the second trace must be provided.\n');
    return
else
    ds = input.ds;
end;

if isfield(input,'tshift')
    tshift = input.tshift;
else
    tshift = [];
end;

Manual_zerotime_Artefact_Flag = false;
if isfield(input,'ext_zt') && ~isempty(input.ext_zt),
    Manual_zerotime_Artefact = input.ext_zt; 
    Manual_zerotime_Artefact_Flag = true;
end;

if ~isfield(input,'display') || isempty(input.display),
    displayed = true;
else
    displayed = input.display;
end;

% Check and default of expert mode parameters

if ~isfield(input,'bg_mode') || isempty(input.bg_mode),
    options.bg_mode = '3D';
else
    options.bg_mode = input.bg_mode;
end;

if ~isfield(input,'cutdata') || isempty(input.cutdata),
    cutdata = 1;
else
    cutdata = input.cutdata;
end;

if ~isfield(input,'t4') || isempty(input.t4),
    t4 = [];
else
    t4 = input.t4;
end;

if ~isfield(input,'tr4') || isempty(input.tr4),
    t4 = [];
    tr4 = [];
else
    tr4 = input.tr4;
end;

if ~isfield(input,'exp_depth'),
    options.exp_depth = 0.3;
else
    options.exp_depth = input.exp_depth;
end;

if ~isfield(input,'fit_depth'),
    options.fit_depth = 1;
else
    options.fit_depth = input.fit_depth;
end;

if ~isfield(input,'cut4p') || isempty(input.cut4p),
    input.cut4p = 0;
end;

if ~isfield(input,'interp') || isempty(input.interp),
    input.interp = 0;
end;

% flag for fitting amplitude and time shift of artefact 
% by minimizing difference of the two corrected traces
if ~isfield(input,'sec_fit') || isempty(input.sec_fit),
    second_fit = 0;
else
    second_fit = input.sec_fit; 
end;

% Check programmer mode parameters

if ~isfield(input,'degree') || isempty(input.degree),
    degree = 3;
else
    degree = input.degree;
end;

dt=t1(2)-t1(1);

cutin=floor(cutdata*length(t1));
ffan=ffan(1:cutin);
ffbn=ffbn(1:cutin);
t1=t1(1:cutin);

interpolate_factor = 1;
% Data interpolation
if input.interp,
    dtf = 0.5;
    interpolate_factor = dt/dtf;
    ds = interpolate_factor*ds;
    t2 = min(t1):dtf:max(t1);
    ffan = interp1(t1,ffan,t2,'spline',ffan(1));
    ffbn = interp1(t1,ffbn,t2,'spline',ffbn(1));
    t1=t2;
    dt=t1(2)-t1(1);
end;

% Determine signal width in time domain
deer0 = (ffan+ffbn)/2;
hp = get_half_decay(t1,deer0);
opt_ds = round(2*hp/(3*interpolate_factor));
fprintf(1,'The suggested ds value would have been %i\n',opt_ds);

    
diff=ffan-ffbn; % difference of the two traces, 5-pulse signal is removed

if displayed,
    % Display the two noisy traces
    figure(1); clf;
    plot(t1,ffan,'k');
    hold on;
    plot(t1,ffbn,'r');
    title('The two noisy traces (input data)');
    set(gca,'FontSize',10);
    % Display difference trace (numerical derivative of the artefact)
    figure(2); clf;
    plot(t1,diff,'k');
    hold on
    plot([min(t1) max(t1)],[0 0],'g');
    title('Difference trace (artefact derivative)');
    set(gca,'FontSize',10);
end;

% generate a first estimate for the artefact signal by cumulative sums
% of the difference signal, points 1, 1+ds, 1+2ds, ... are summed to
% obtain the 'integrated' point 1, points 2, 2+ds, 2+2ds... for
% integrated point 2 and so on
% high-frequency noise at frequencies larger than 1/(2*ds*dt) will be
% repetitive in the resulting 'integrated' signal
csdiff=diff;
for k=1:length(diff)-ds,
    csdiff(k)=diff(k);
    k2=k+ds;
    while k2<=length(diff),
        csdiff(k)=csdiff(k)+diff(k2);
        k2=k2+ds;
    end;
end;


% suppress high-frequency noise by low-pass filtering in frequency domain
[nu,spc]=dft(t1,csdiff); % discrete Fourier transformation

nx=round(2*length(diff)/(ds+2)); % cutoff point for low-pass filter
filtered=[real(spc(1:nx)) zeros(1,length(diff)-nx)]; % remove high frequencies
[t,filtcorr]=dft(nu,filtered); % inverse FT
filtcorr=2*real(filtcorr)/length(diff); % compensate amplitude change

if isempty(tshift)
    [~,pma] = max(filtcorr);
    tshift = t1(pma);
end;
tmi = tshift - ds*dt; % lower limit for the time range in which the artefact
% zero time is expected in the first trace
tma = tshift + ds*dt; % lower limit for the time range in which the artefact
% zero time is expected in the first trace

% caused by DFT/inverse DFT

% display
if displayed,
    % theoretical artefact signal and reconstructed unfiltered
    % artefact signal
    figure(3); clf;
    hold on;
    plot(t1,csdiff,'r');
    plot(t1,filtcorr,'k');
    title('Initial estimate (red) and denoised artefact (black)');
    set(gca,'FontSize',10);
    % spectral representation of unfiltered signal and filtered signal
    figure(4); clf;
    plot(real(spc),'k');
    hold on;
    plot([nx,nx],[min(real(spc)),max(real(spc))],'b:');
    plot(filtered,'r');
    title('Spectrum of initial estimate (black), cutoff (dotted blue), filtered (red)');
    set(gca,'FontSize',10);
end;
    
    
% Determine zero time of the artefact by fitting the maximum in a
% reasonable time interval around the expected time shift
t0 = maxtime(t,filtcorr,ds,tmi,tma);
fprintf(1,'Automatically determined artefact zero time: %4.1f ns\n',t0);
if Manual_zerotime_Artefact_Flag;
    t0 = Manual_zerotime_Artefact;
    fprintf(1,'Zerotime of the artefact was externally set to %4.1f ns\n',t0);
end
if isempty(t0), % the fit may have failed if the filtered artefact was very
    % noisy and the time range too broad
    t0 = mean(t);
    fprintf(1,'Zerotime fit failed and zero time set to center (%4.1f ns)\n',t0);
end;
    
% symmetrize the trace
ta = t - t0; % time axis that is zero at artefact maximum
if abs(ta(1))<abs(ta(end)), % data part that is known for both negative
    % and positive times
    tsym = linspace(ta(1),-ta(1),length(t));
else
    tsym = linspace(-ta(end),ta(end),length(t));
end;
fcs = interp1(ta,filtcorr,tsym); % interpolate to time axis where zero
% time is at the center
fcs = (fcs + fliplr(fcs))/2; % symmetrize the filtered artefact
    
corra = ffan-filtcorr; % first correction of first trace

tb = ta-ds*dt; % shifted time axis for artefact in second trace (CHEEECK!!!!)
fcb = interp1(tb,filtcorr,ta,'linear',0); % interpolate artefact for second trace
corrb = ffbn-fcb; % corrected form of second trace
corrected = (corra+corrb)/2; % mean corrected trace

% display first corrections of both traces and their mean
if displayed,
    figure(5); clf;
    plot(t1,corra,'b');
    hold on;
    plot(t1,corrb,'r');
    plot(t1,corrected,'k');
    title('Corrected first (blue) and second (red) trace and their mean (black)');
    set(gca,'FontSize',10);
end;
    
% generate estimate of artefact signal from corrected trace
iterate=[fliplr(corrected(2:end)) corrected]; % mirror extension
ti=-(length(corrected)-1)*dt:dt:(length(corrected)-1)*dt; % time axis for iterated data
iterateb=interp1(ti+t0,iterate,tsym+t0); % interpolate estimate to
% the time axis of the filtered symmetrized
% original artefact estimate

sc = fit_lin(iterateb,fcs,degree); % fit the artefact amplitude
iterate1 = sc*interp1(ti+t0,iterate,t1); % time shift and scaled artefact estimate for first trace
iterate2 = sc*interp1(ti+t0-ds*dt,iterate,t1); % time shift and scaled artefact estimate for second trace

if displayed,
    % display filtered artefact, its symmetrized version, the
    % improved estimate, and the determined zero time
    figure(6); clf;
    hold on;
    plot(t1,filtcorr,'b');
    plot(tsym+t0,fcs,'m');
    plot(t1,iterate1,'k');
    title('Denoised (blue), symmetrized (magenta) and improved (black) estimate');
    set(gca,'FontSize',10);
end;
    
% further improve the artefact estimate by self-consistent interation
sc0=0; % no previous scaling factor was known
iterations=0; % count number of iterations
if displayed
    fprintf(1,' sc initial = %0.3f \n',sc);
end;
while abs(sc0-sc)/sc > 1e-3 && iterations < 1000, % amplitude should be self-consistent to better than 0.1%
    % but infinite loop should be avoided if this cannot be achieved
    sc0 = sc; % store previous amplitude factor
    iterations = iterations + 1; % increment iteration counter
    corra = ffan-iterate1; % new corrected trace 1 with current artefact estimate
    corrb = ffbn-iterate2; % new corrected trace 2 with current artefact estimate
    corrected = (corra+corrb)/2; % mean corrected trace
    iterate = [fliplr(corrected(2:end)) corrected]; % symmetric version
    ti = -(length(corrected)-1)*dt:dt:(length(corrected)-1)*dt; % time axis for iterated data
    iterateb = interp1(ti+t0,iterate,tsym+t0); % interpolate to time axis of filtered, symmetrized
    % experimental estimate
    sc = fit_lin(iterateb,fcs,degree); % fit amplitude to the original filtered and symmetrized artefact estimate
    iterate1 = sc*interp1(ti+t0,iterate,t1); % new estimate
    iterate2 = sc*interp1(ti+t0-ds*dt,iterate,t1); % time shift and scaled artefact estimate
end;
if displayed
    fprintf(' sc self-consistent = %0.3f after %i iterations\n',sc,iterations);
end;
corra = ffan-iterate1; % new correction with current artefact estimate
corrb = ffbn - iterate2; % new correction with current artefact estimate
corrected = (corra+corrb)/2; % mean corrected trace
    
if second_fit,
    % Further fitting by minimization of the difference between the two
    % corrected traces, this is only minimally tested
    clear rmsd_corr_diff % clear the fit function, so that its persistent variables are initialized
    rmsd0 = sqrt((corra-corrb).^2/length(corra)); % initial root mean square deviation
    v0 = [sc,t0]; % initial amplitude and zero time
    v = fminsearch(@rmsd_corr_diff,v0,[],ffan,ffbn,rmsd0,corrected,dt,t1,ds);
    
    sc=v(1);
    t0=v(2);
    if displayed
        fprintf(1,'t0 and sc second fit are: %0.2f %0.3f \n',t0,sc);
    end;
    ni = ceil(log(1e-3)/log(sc)); % dampen error by 10^-3
    if ni < 1, % initially determined scaling factor is larger than 1
        fprintf(2,'ERROR: Scaling factor is larger than 1. Correction failed.\n');
        time = [];
        corrected = [];
        return
    end;
    for ki = 1:ni, % self-consistency loop
        iterate = [fliplr(corrected(2:end)) corrected]; % symmetric version
        iterate1 = sc*interp1(ti+t0,iterate,t1); % new estimate
        iterate2 = sc*interp1(ti+t0-ds*dt,iterate,t1); % time shift and scaled artefact estimate
        corra = ffan-iterate1; % new corrected trace 1 with current artefact estimate
        corrb = ffbn-iterate2; % new corrected trace 2 with current artefact estimate
        corrected = (corra+corrb)/2; % mean corrected trace
    end;
end;
    
if displayed,
    % display original filtered artefact estimate and final artfeact
    % estimate
    figure(7); clf;
    hold on;
    plot(t1,filtcorr,'k');
    plot(t1,iterate1,'r');
    title('Artefact estimates: denoised (black) and final (red)');
    set(gca,'FontSize',10);
end;

if input.compare4DEER_flag;   
% Fit 4-pulse DEER background to 5-pulse DEER, only done if such a trace is
% given
if ~isempty(tr4) && ~isempty(t4)
    orig=tr4; 
    corrected_norm = corrected/max(corrected);

    % length adjustment for corrected and ref 4 deer traces
    [minlength , ~] = min([length(orig), length(corrected_norm)]);
    if isfield(input,'cut4p') 
        minlength = minlength - input.cut4p; % reduced minimum length if 4-DEER contains 2+1 artefact at the end 
    end;
    corrected_norm_fit = corrected_norm(1:minlength);
    t1fit = t1(1:minlength);
    orig_fit = orig(1:minlength);

    pake_base = load('pake_base40.mat'); % deeranalysis's thing: provides r, t and kernel variables
    options.Pake_t = pake_base.t;
    options.Pake_r = pake_base.r;
    base = pake_base.base;
    options.Pake_kernel = base-ones(size(base)); % kernel format for pcf2deer
    [orig_adjusted,~,bckg_k,mod_depth,dim] = fit_primary_loc(options,t1fit,corrected_norm_fit,orig_fit);

    lw = 2; 
    if displayed,
        h=figure(8); clf;
        hold on
        plot(t4/1000,orig,'Color',[0 0 0],'LineWidth',lw);
        plot(t1/1000,corrected_norm,'Color',[31/250 64/250 122/250],'LineWidth',lw);
        plot(t1fit/1000,orig_adjusted,'Color',[0.8 0 0],'LineWidth',lw);
        legend('original 4-pulse trace','corrected 5pulse trace','orig. 4-pulse fit to 5-pulse')
        xlim([-200/1000 1.1*max(t1)/1000])
        h.CurrentAxes.FontSize = 12;
        ylabel('Norm. intensity'); xlabel('\itt \rm(\mus)')
    end;

    if isfield(input,'saveASCII') || saveASCII==1    
        orig4p=[t4' orig'];
        oname1=['orig4p' '.dat'];
        save(oname1,'orig4p','-ascii');

        orig4p_adj=[t1fit' orig_adjusted'];
        oname1=['orig4p_adj' '.dat'];
        save(oname1,'orig4p_adj','-ascii');

        orig5p=[t1' ffan'];
        oname1=['orig5p' '.dat'];
        save(oname1,'orig5p','-ascii');

        cleaned5p=[t1' corrected_norm'];
        oname1=['cleaned5p' '.dat'];
        save(oname1,'cleaned5p','-ascii');    
    end;
    if displayed,
        fprintf(1,'Background dimension %0.3f, decay constant %0.3f, and modulation depth %0.3f\n',dim,bckg_k,mod_depth);
    end;
end
end;

time = t1;

function tmax = maxtime(t,data,dp,tmi,tma)
% function tmax = maxtime(t,data,dp)
%
% Determines time of the maximum of a function by a polynomial fit in a
% range of +/-0.75 dp data points around the absolute maximum
%
% t     time axis of original data
% data  data trace
% dp    data points determining search range
% tmi   minimum time for maximum location (optional)
% tma   maximum time for maximum location (optional)
% tmax  time corresponding to the maximum
%
% G. Jeschke, 2015

degree = 3; % degree of the fit polynomial

if ~exist('tmi','var'), tmi = t(1); end;
if ~exist('tma','var'), tma = t(end); end;

ds = round(3*dp/4);
[~,pmi] = min(abs(t-tmi));
[~,pma] = min(abs(t-tma));
[~,poi] = max(data(pmi:pma));
poi = pmi+poi-1;
if poi < ds+1 || poi > length(t)-ds,
    tmax = [];
    return
end;
[p,~,mu] = polyfit(t(poi-ds:poi+ds),data(poi-ds:poi+ds),degree);
tfine = linspace(t(poi-ds),t(poi+ds),20*ds);
y = polyval(p,tfine,[],mu);
[~,poi] = max(y);
tmax = tfine(poi);

function sc = fit_lin(iterate,filtcorr,degree)

if ~exist('degree','var'),
    degree = 0; % degree of the baseline polynomial
end;

t = 1:length(filtcorr);
[p,~,mu] = polyfit(t,filtcorr,degree);
bsl = polyval(p,t,[],mu);
filtcorr = filtcorr - bsl;

[p,~,mu] = polyfit(t,iterate,degree);
bsl2 = polyval(p,t,[],mu);
iterateb = iterate-bsl2;

sc0 = sum(filtcorr.^2)/sum(filtcorr.*iterateb);

v0 = [sc0,0];
v = fminsearch(@rmsd_lin,v0,[],iterateb,filtcorr);
sc = v(1);
% iterate1 = sc*iterate;

function rmsd = rmsd_lin(v,iterate,filtcorr)

diff = v(1)*iterate-filtcorr+v(2);
rmsd = sqrt(sum(diff.^2));

function [hp,th] = get_half_decay(t,data,options)
% Determines time, where the modulation in the signal has decayed to half
% the initial value

% expoential background fit of normalized data
tr = data/max(data);
ps = round(length(t)/4);
poly = polyfit(t(ps:end),log(tr(ps:end)),1); % linear fit of logarithm
bckg = exp(polyval(poly,t)); % background is exponential of that
mod_depth = 1-bckg(1);
hd = 1 - mod_depth/2;
[~,hp] = min(abs(tr-hd));
th = t(hp);
if exist('options','var') && options.verbose,
    figure;
    plot(t,tr,'k');
    hold on;
    plot(t,bckg,'r');
    plot([th,th],[0,1],'b:');
    fprintf(1,'Time of half decay is : %4.1f ns\n',th);
end;

function [deer,bckg,bckg_k,mod_depth,dim,rmsd] = fit_primary_loc(options,texp,vexp,ff)

% Y. Polyhach 14/05/12: local version of fit_primary.m from MMM2011_1

% Fits the primary DEER data defined by time axis texp and data trace vexp
% with a given simulated form factor ff by varying modulation depth and
% background decay constant
% for a stretched exponential background (model 'fractional'), background
% dimension is also varied
% the background model is specified in handles:
%
% deer  simulated primary data (fit to vexp) 


% define bckg model for fitting
if isempty(options.exp_depth)
    depth0 = 0.3;
else
    depth0 = options.exp_depth;
end
 
poly = polyfit(texp,log(vexp),1); % linear fit of logarithm
v0 = [depth0 -poly(1)];
switch options.bg_mode,
    case 'fractal',
        if options.fit_depth % any numerical or string value for fit_depth will do
            v0=[depth0 -poly(1) 3];
            [v1,rmsd]=fminsearch(@(v0) rms_stretched_MMM(v0,texp,vexp,ff), v0);
            mod_depth=v1(1);
            v1=v1(2:3);
        else
            v0=[-poly(1) 3];
            [v1,rmsd]=fminsearch(@(v0) rms_stretched_MMM(v0,texp,vexp,ff,depth0), v0);
            mod_depth=depth0;
        end;
        bckg=(1-mod_depth)*decay_stretched_MMM(v1,texp);
        bckg_k=v1(1);
        dim=v1(2);
    case '2D',
        distr=options.Pake_r; % 2D distribution has probability proportional to r
        distr=0.01*distr/sum(distr);
        logB=distr*options.Pake_kernel;
        if options.fit_depth,
            [v1,rmsd]=fminsearch(@(v0) rmsnD_MMM(v0,texp,vexp,ff,options.Pake_t,logB), v0);
            mod_depth=v1(1);
            v1=v1(2);
        else
            v0=v0(2);
            [v1,rmsd]=fminsearch(@(v0) rmsnD_MMM(v0,texp,vexp,ff,options.Pake_t,logB,depth0), v0);
            mod_depth=depth0;
        end;
        bckg=(1-mod_depth)*decaynD_MMM(v1,texp,options.Pake_t,logB);
        bckg_k=v1;
        dim=2;
    case '3D',
        distr=options.Pake_r.*options.Pake_r;
        distr=0.01*distr/sum(distr);
        logB=distr*options.Pake_kernel;
        if options.fit_depth,
            [v1,rmsd]=fminsearch(@(v0) rmsnD_MMM(v0,texp,vexp,ff,options.Pake_t,logB), v0);
            mod_depth=v1(1);
            v1=v1(2);
        else
            v0=v0(2);
            [v1,rmsd]=fminsearch(@(v0) rmsnD_MMM(v0,texp,vexp,ff,options.Pake_t,logB,depth0), v0);
            mod_depth=depth0;
        end;
        bckg=(1-mod_depth)*decaynD_MMM(v1,texp,options.Pake_t,logB);
        bckg_k=v1;
        dim=3;
    otherwise, % this includes background none (yepo: returns ff essentially only)
        if options.fit_depth,
            [mod_depth,rmsd]=fminsearch(@(depth0) rmsd_depth(depth0,vexp,ff), depth0);
        else
            mod_depth=depth0;
            sim=ff*mod_depth+ones(size(ff))*(1-mod_depth);
            diff=sim-vexp;
            rmsd=sqrt((diff.*diff)/length(diff));
        end;
        bckg_k=0;
        bckg=(1-mod_depth)*ones(size(vexp));
        dim=3;
end;
bckg=real(bckg);

deer=ff*mod_depth+ones(size(ff))*(1-mod_depth);
deer=bckg.*deer/max(bckg);
deer=deer./max(deer);

function rms = rmsnD_MMM(v,texp,vexp,ff,t0,logB,mod_depth)
%rmsnd_MMM	Root mean square error of function exp(-k*t^(n/3)).	
%  Parameter: v(1) Modulation depth, v(2) time constant
%	Copyright (c) 2009 by Gunnar Jeschke

if v(1)<0, rms=1.0e6; return; end;
if length(v)>1,
    if v(2)<0, rms=1.0e6; return; end;
    if v(1)>1, rms=1.0e6; return; end;
    mod_depth=v(1);
    v=v(2);
end;
bckg=decaynD_MMM(v,texp,t0,logB);

deconvoluted=mod_depth*ff+(1-mod_depth)*ones(size(ff));
sim=deconvoluted.*bckg;

diff=sim-vexp;
rms=sqrt(sum(diff.^2)/length(diff));

function rms = rms_stretched_MMM(v,texp,vexp,ff,mod_depth)
%RMS_stretched_MMM	Root mean square error of function exp(-k*t^ksi).	
%  Parameter: v(1) Modulation depth, v(2) time constant k, v(3) ksi
% mod_depth is an optional argument, if present, depth is not fitted
%	Copyright (c) 2009 by Gunnar Jeschke

if v(1)<0, rms=1.0e6; return; end;
if v(2)<0, rms=1.0e6; return; end;
if length(v)>2,
    if v(3)<0, rms=1.0e6; return; end; 
    if v(1)>1, rms=1.0e6; return; end;
end;

if nargin>4,
    bckg=decay_stretched_MMM(v,texp);
else
    bckg=decay_stretched_MMM(v(2:3),texp);
    mod_depth=v(1);
end;
deconvoluted=mod_depth*ff+(1-mod_depth)*ones(size(ff));
sim=deconvoluted.*bckg;

diff=sim-vexp;
rms=sqrt(sum(diff.^2)/length(diff));

function sim=decaynD_MMM(v,texp,t0,logB)
% Computes DEER decay for n-dimensional homogeneous distribution
% type of distribution is provided in argument logB

% v(1)  decay constant
% texp  abscissa values for result (vector)
% t0    abscissa values corresponding to logb

sim0=exp(16000*v(1)*logB);
sim=interp1(t0,sim0,texp,'pchip');

function sim=decay_stretched_MMM(v,x)
% Computes stretched exponential decay function
%
% v(1)  decay constant
% v(2)  3 ksi, where ksi is the stretch exponent
% x     abscissa values (vector)
%
sim=exp(-v(1)*x.^(v(2)/3));

function rmsd=rmsd_depth(depth,vexp,ff)

if depth<0,
    rmsd=1e6;
    return
end;
if depth>1,
    rmsd=1e6;
    return
end;
sim=ff*depth+ones(size(ff))*(1-depth);
diff=sim-vexp;
rmsd=sqrt(sum(diff.^2)/length(diff));


